#include "StdAfx.h"
#include "ChatRoomManager.h"
#include "ChatRoomView.h"

void CChatRoomManager::NotifyView( CChatRoom* pChatRoom, ChatRoomAction Action )
{
	if(m_pView != NULL)
	{
		switch(Action)
		{
		case AddAction:
			m_pView->OnAdd(pChatRoom);
			break;
		case UpdateAction:
			m_pView->OnUpdate(pChatRoom, this);
			break;
		}	
	}
}

void CChatRoomManager::NotifyView( int nRoomID, ChatRoomAction Action )
{
	if(m_pView != NULL)
	{
		switch(Action)
		{
		case RemoveAction:
			m_pView->OnRemove(nRoomID);
			break;
		}	
	}
}

bool CChatRoomManager::Add( CChatRoom* pChatRoom )
{
	CAutoLock lock(&m_csChatRoomList);

	if(NULL == pChatRoom) return false;

	ITListChatRoom it = find_if(m_listChatRoom.begin(), m_listChatRoom.end(), CRoomValueFinder(pChatRoom));
	if( it == m_listChatRoom.end() )
	{
		m_listChatRoom.push_back(pChatRoom);
	}

	NotifyView(pChatRoom, AddAction);
    return true;
}

bool CChatRoomManager::Remove( int nRoomID )
{
	CAutoLock lock(&m_csChatRoomList);

	ITListChatRoom it = remove_if(m_listChatRoom.begin(), m_listChatRoom.end(), CRoomValueFinder(nRoomID));
	if( it != m_listChatRoom.end() )
	{
		SAFE_DELETE(*it);
		m_listChatRoom.erase(it);
	}

	NotifyView(nRoomID, RemoveAction);

	return true;
}

bool CChatRoomManager::Update( CChatRoom* pChatRoom, CChatRoomManager* pRoomManager )
{
	CAutoLock lock(&m_csChatRoomList);

	ITListChatRoom it = find_if(m_listChatRoom.begin(), m_listChatRoom.end(), CRoomValueFinder(pChatRoom));
	if( it != m_listChatRoom.end() )
	{
		(*it)->Copy(*pChatRoom);
	}

	NotifyView(pChatRoom, UpdateAction);

	return true;
}

CChatRoom* CChatRoomManager::GetChatRoom( int nRoomID )
{
	CAutoLock lock(&m_csChatRoomList);

	ITListChatRoom it = remove_if(m_listChatRoom.begin(), m_listChatRoom.end(), CRoomValueFinder(nRoomID));
	if( it != m_listChatRoom.end() )
	{
		return *it;
	}

	return NULL;
}
